%==========================================================================
%               Reading the .h5 file created according to the
%                    JONAS Exchange Data Format Proposal
%==========================================================================
%Function description:
%This function permit to read a ".h5" file taking into account the parameters
%defined at the JONAS Exchange Data Format proposal.
%
%See the following list of parameters:
%format_version 
%author 
%institution
%country_code
%contact
%start_date
%end_date
%date_of_creation
%purpose
%data_uuid
%data_type
%comments
%expsetup
%recorder_manufacturer
%recorder_serial_number
%recorder_model
%builtin_hydrophone
%hydrophone_manufacturer
%hydrophone_sensitivity
%hydrophone_serial_number
%hydrophone_model
%calibration_frequency_count
%calibration_datetime
%calibration_factor
%calibration_procedure
%reference_frequencies
%ais_database
%source_levels
%bathymetry_database
%emperature_database
%salinity_database
%sound_speed_profile_model
%propagation_model
%numeric_data_calibration
%hydrophone_count
%total_number_of_grid_points
%longitude
%latitude
%depth
%frequency_count
%frequency_band_definition
%frequency
%time_duty_on
%time_duty_off
%time_count
%time
%averaging_time
%spl_values
%percentile_list
%percentile_count
%percentile
%--------------------------------------------------------------------------
%
%The complete field description can be seen at the JONAS Deliverable D4.3 
%available at: xxxxxxxxxx
%
%Calling the function examples:
%
%DATA=read_edf('filename.h5')
%--------------------------------------------------------------------------

function DATA=read_edf(file_name)

data_type=h5readatt(file_name,'/','data_type')

if isempty(data_type)==1
    
    disp('No "data_type" was defined in the EDF file');
    
    DATA=struct('format_version',h5readatt(file_name,'/','format_version'),...
        'author', h5readatt(file_name,'/','author'),...
        'institution', h5readatt(file_name,'/','institution'),...
        'country_code', h5readatt(file_name,'/','country_code'),...
        'contact', h5readatt(file_name,'/','contact'),...
        'start_date', h5readatt(file_name,'/','start_date'),...
        'end_date', h5readatt(file_name,'/','end_date'),...
        'date_of_creation', h5readatt(file_name,'/','date_of_creation'),...
        'purpose', h5readatt(file_name,'/','purpose'),...
        'data_uuid', h5readatt(file_name,'/','data_uuid'),...
        'data_type', h5readatt(file_name,'/','data_type'),...
        'comments', h5readatt(file_name,'/','comments'),...
        'expsetup',[],...
        'recorder_manufacturer', [],...
        'recorder_serial_number', [],...
        'recorder_model', [],...
        'builtin_hydrophone', [],...
        'hydrophone_manufacturer', [],...
        'hydrophone_sensitivity', [],...
        'hydrophone_serial_number', [],...
        'hydrophone_model', [],...
        'calibration_frequency_count', [],...
        'calibration_datetime', [],...
        'calibration_factor', [],...
        'calibration_procedure', [],...
        'reference_frequencies', [],...
        'ais_database', [],...
        'source_levels', [],...
        'bathymetry_database', [],...
        'temperature_database', [],...
        'salinity_database', [],...
        'sound_speed_profile_model', [],...
        'propagation_model', [],...
        'numeric_data_calibration', [],...
        'hydrophone_count', [],...
        'total_number_of_grid_points', [],...
        'longitude', h5read(file_name,'/ambient_noise_dataset/position/longitude'),...
        'latitude', h5read(file_name,'/ambient_noise_dataset/position/latitude'),...
        'depth', h5read(file_name,'/ambient_noise_dataset/position/depth'),...
        'frequency_count', h5read(file_name,'/ambient_noise_dataset/frequency/frequency_count'),...
        'frequency_band_definition', '1/3octave-band levels',...
        'frequency', h5read(file_name,'/ambient_noise_dataset/frequency/frequency'),...
        'time_duty_on', [],...
        'time_duty_off', [],...
        'time_count', h5read(file_name,'/ambient_noise_dataset/time/time_count'),...
        'time', h5read(file_name,'/ambient_noise_dataset/time/time'),...
        'averaging_time', [],...
        'spl_values', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels/spl_values'),...
        'percentile_list', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_count'),...
        'percentile_count', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_list'),...
        'percentile', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_values'));
    
elseif strcmp(data_type,'Experimental')
    
    disp('Experimental "data_type" was defined in the EDF file');
    
    DATA=struct('format_version',h5readatt(file_name,'/','format_version'),...
        'author', h5readatt(file_name,'/','author'),...
        'institution', h5readatt(file_name,'/','institution'),...
        'country_code', h5readatt(file_name,'/','country_code'),...
        'contact', h5readatt(file_name,'/','contact'),...
        'start_date', h5readatt(file_name,'/','start_date'),...
        'end_date', h5readatt(file_name,'/','end_date'),...
        'date_of_creation', h5readatt(file_name,'/','date_of_creation'),...
        'purpose', h5readatt(file_name,'/','purpose'),...
        'data_uuid', h5readatt(file_name,'/','data_uuid'),...
        'data_type', h5readatt(file_name,'/','data_type'),...
        'comments', h5readatt(file_name,'/','comments'),...
        'expsetup', h5readatt(file_name,'/analysis_metadata/experimental','expsetup'),...
        'recorder_manufacturer', h5readatt(file_name,'/analysis_metadata/experimental/recorder','recorder_manufacturer'),...
        'recorder_serial_number', h5readatt(file_name,'/analysis_metadata/experimental/recorder','recorder_serial_number'),...
        'recorder_model', h5readatt(file_name,'/analysis_metadata/experimental/recorder','recorder_model'),...
        'builtin_hydrophone', h5readatt(file_name,'/analysis_metadata/experimental/recorder','builtin_hydrophone'),...
        'hydrophone_manufacturer', h5readatt(file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_manufacturer'),...
        'hydrophone_sensitivity', h5readatt(file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_sensitivity'),...
        'hydrophone_serial_number', h5readatt(file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_serial_number'),...
        'hydrophone_model', h5readatt(file_name,'/analysis_metadata/experimental/hydrophone','hydrophone_model'),...
        'calibration_frequency_count', h5read(file_name,'/analysis_metadata/experimental/calibration/calibration_frequency_count'),...
        'calibration_datetime', h5read(file_name,'/analysis_metadata/experimental/calibration/calibration_datetime'),...
        'calibration_factor', h5read(file_name,'/analysis_metadata/experimental/calibration/calibration_factor'),...
        'calibration_procedure', h5read(file_name,'/analysis_metadata/experimental/calibration/calibration_procedure'),...
        'reference_frequencies', h5read(file_name,'/analysis_metadata/experimental/calibration/reference_frequencies'),...
        'ais_database', [],...
        'source_levels', [],...
        'bathymetry_database', [],...
        'temperature_database', [],...
        'salinity_database', [],...
        'sound_speed_profile_model', [],...
        'propagation_model', [],...
        'numeric_data_calibration', [],...
        'hydrophone_count', h5read(file_name,'/ambient_noise_dataset/position/hydrophone_count'),...
        'total_number_of_grid_points', [],...
        'longitude', h5read(file_name,'/ambient_noise_dataset/position/longitude'),...
        'latitude', h5read(file_name,'/ambient_noise_dataset/position/latitude'),...
        'depth', h5read(file_name,'/ambient_noise_dataset/position/depth'),...
        'frequency_count', h5read(file_name,'/ambient_noise_dataset/frequency/frequency_count'),...
        'frequency_band_definition', '1/3octave-band levels',...
        'frequency', h5read(file_name,'/ambient_noise_dataset/frequency/frequency'),...
        'time_duty_on', h5readatt(file_name,'/ambient_noise_dataset/time','time_duty_on'),...
        'time_duty_off', h5readatt(file_name,'/ambient_noise_dataset/time','time_duty_off'),...
        'time_count', h5read(file_name,'/ambient_noise_dataset/time/time_count'),...
        'time', h5read(file_name,'/ambient_noise_dataset/time/time'),...
        'averaging_time', h5readatt(file_name,'/ambient_noise_dataset/sound_pressure_levels','averaging_time'),...
        'spl_values', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels/spl_values'),...
        'percentile_list', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_count'),...
        'percentile_count', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_list'),...
        'percentile', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_values'));
    
elseif strcmp(data_type,'Numerical')
    
    disp('Numerical "data_type" was defined in the EDF file');
    
    DATA=struct('format_version',h5readatt(file_name,'/','format_version'),...
        'author', h5readatt(file_name,'/','author'),...
        'institution', h5readatt(file_name,'/','institution'),...
        'country_code', h5readatt(file_name,'/','country_code'),...
        'contact', h5readatt(file_name,'/','contact'),...
        'start_date', h5readatt(file_name,'/','start_date'),...
        'end_date', h5readatt(file_name,'/','end_date'),...
        'date_of_creation', h5readatt(file_name,'/','date_of_creation'),...
        'purpose', h5readatt(file_name,'/','purpose'),...
        'data_uuid', h5readatt(file_name,'/','data_uuid'),...
        'data_type', h5readatt(file_name,'/','data_type'),...
        'comments', h5readatt(file_name,'/','comments'),...
        'exsetup',[],...
        'recorder_manufacturer',[],...
        'recorder_serial_number',[],...
        'recorder_model',[],...
        'builtin_hydrophone',[],...
        'hydrophone_manufacturer',[],...
        'hydrophone_sensitivity',[],...
        'hydrophone_serial_number',[],...
        'hydrophone_model',[],...
        'calibration_frequency_count',[],...
        'calibration_datetime',[],...
        'calibration_factor',[],...
        'calibration_procedure',[],...
        'reference_frequencies',[],...
        'ais_database', h5readatt(file_name,'/analysis_metadata/numerical_model','ais_database'),...
        'source_levels', h5read(file_name,'/analysis_metadata/numerical_model/source_levels'),...
        'bathymetry_database', h5readatt(file_name,'/analysis_metadata/numerical_model','bathymetry_database'),...
        'temperature_database', h5readatt(file_name,'/analysis_metadata/numerical_model','temperature_database'),...
        'salinity_database', h5readatt(file_name,'/analysis_metadata/numerical_model','salinity_database'),...
        'sound_speed_profile_model', h5readatt(file_name,'/analysis_metadata/numerical_model','sound_speed_profile_model'),...
        'propagation_model', h5readatt(file_name,'/analysis_metadata/numerical_model','propagation_model'),...
        'numeric_data_calibration', h5readatt(file_name,'/analysis_metadata/numerical_model','numeric_data_calibration'),...
        'hydrophone_count', [],...
        'total_number_of_grid_points', h5read(file_name,'/ambient_noise_dataset/position/total_number_of_grid_points'),...
        'longitude', h5read(file_name,'/ambient_noise_dataset/position/longitude'),...
        'latitude', h5read(file_name,'/ambient_noise_dataset/position/latitude'),...
        'depth', h5read(file_name,'/ambient_noise_dataset/position/depth'),...
        'frequency_count', h5read(file_name,'/ambient_noise_dataset/frequency/frequency_count'),...
        'frequency_band_definition', [],...
        'frequency', h5read(file_name,'/ambient_noise_dataset/frequency/frequency'),...
        'time_duty_on', [],...
        'time_duty_off', [],...
        'time_count', h5read(file_name,'/ambient_noise_dataset/time/time_count'),...
        'time', h5read(file_name,'/ambient_noise_dataset/time/time'),...
        'averaging_time', [],...
        'spl_values', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels/spl_values'),...
        'percentile_list', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_count'),...
        'percentile_count', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_list'),...
        'percentile', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_values'));
    
else
    
    disp('No valid "data_type" was defined in the EDF file');
    disp('"data_type" should be defined has "Experimental" or "Numerical" in the EDF metadata file');
    
    DATA=struct('format_version',h5readatt(file_name,'/','format_version'),...
        'author', h5readatt(file_name,'/','author'),...
        'institution', h5readatt(file_name,'/','institution'),...
        'country_code', h5readatt(file_name,'/','country_code'),...
        'contact', h5readatt(file_name,'/','contact'),...
        'start_date', h5readatt(file_name,'/','start_date'),...
        'end_date', h5readatt(file_name,'/','end_date'),...
        'date_of_creation', h5readatt(file_name,'/','date_of_creation'),...
        'purpose', h5readatt(file_name,'/','purpose'),...
        'data_uuid', h5readatt(file_name,'/','data_uuid'),...
        'data_type', h5readatt(file_name,'/','data_type'),...
        'comments', h5readatt(file_name,'/','comments'),...
        'expsetup',[],...
        'recorder_manufacturer', [],...
        'recorder_serial_number', [],...
        'recorder_model', [],...
        'builtin_hydrophone', [],...
        'hydrophone_manufacturer', [],...
        'hydrophone_sensitivity', [],...
        'hydrophone_serial_number', [],...
        'hydrophone_model', [],...
        'calibration_frequency_count', [],...
        'calibration_datetime', [],...
        'calibration_factor', [],...
        'calibration_procedure', [],...
        'reference_frequencies', [],...
        'ais_database', [],...
        'source_levels', [],...
        'bathymetry_database', [],...
        'temperature_database', [],...
        'salinity_database', [],...
        'sound_speed_profile_model', [],...
        'propagation_model', [],...
        'numeric_data_calibration', [],...
        'hydrophone_count', [],...
        'total_number_of_grid_points', [],...
        'longitude', h5read(file_name,'/ambient_noise_dataset/position/longitude'),...
        'latitude', h5read(file_name,'/ambient_noise_dataset/position/latitude'),...
        'depth', h5read(file_name,'/ambient_noise_dataset/position/depth'),...
        'frequency_count', h5read(file_name,'/ambient_noise_dataset/frequency/frequency_count'),...
        'frequency_band_definition', '1/3octave-band levels',...
        'frequency', h5read(file_name,'/ambient_noise_dataset/frequency/frequency'),...
        'time_duty_on', [],...
        'time_duty_off', [],...
        'time_count', h5read(file_name,'/ambient_noise_dataset/time/time_count'),...
        'time', h5read(file_name,'/ambient_noise_dataset/time/time'),...
        'averaging_time', [],...
        'spl_values', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels/spl_values'),...
        'percentile_list', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_count'),...
        'percentile_count', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_list'),...
        'percentile', h5read(file_name,'/ambient_noise_dataset/sound_pressure_levels_stats/percentile_values'));

end

